@@ -6,13 +6,13 @@ import Data.List {- base -}
 import qualified Data.List.Ordered as O {- data-ordlist -}
 import qualified Data.Map as M {- containers -}
 import Data.Maybe {- base -}
-import Data.Monoid {- base -}
 import Data.Ratio {- base -}
 import Safe {- safe -}
 
 import Music.Theory.Function {- hmt -}
 import qualified Music.Theory.List as T {- hmt -}
 import qualified Music.Theory.Math as T {- hmt -}
+import qualified Music.Theory.Ord as T {- hmt -}
 import qualified Music.Theory.Tuple as T {- hmt -}
 
 -- * Types
@@ -26,16 +26,16 @@ type Dseq t a = [(t,a)]
 
 -- | Inter-offset sequence.  The duration is the interval /before/ the
 -- value.  To indicate the duration of the final value /a/ must have
--- an /nil/ (end of sequence) value.
+-- a /nil/ (end of sequence) value.
 type Iseq t a = [(t,a)]
 
 -- | Pattern sequence.  The duration is a triple of /logical/,
 -- /sounding/ and /forward/ durations.
 type Pseq t a = [((t,t,t),a)]
 
--- | Time-point sequence.  To express holes /a/ must have a /empty/
+-- | Time-point sequence.  To express holes /a/ must have an /empty/
 -- value.  To indicate the duration of the final value /a/ must have
--- an /nil/ (end of sequence) value.
+-- a /nil/ (end of sequence) value.
 type Tseq t a = [(t,a)]
 
 -- | Window sequence.  The temporal field is (/time/,/duration/).
@@ -69,6 +69,20 @@ tseq_tspan = seq_tspan id id
 wseq_tspan :: Num t => Wseq t a -> (t,t)
 wseq_tspan = seq_tspan fst (uncurry (+))
 
+-- | Start time of sequence.
+--
+-- > wseq_start [((1,2),'a')] == 1
+-- > wseq_start [] == 0
+wseq_start :: Num t => Wseq t a -> t
+wseq_start = fst . wseq_tspan
+
+-- | End time of sequence.
+--
+-- > wseq_end [((1,2),'a')] == 3
+-- > wseq_end (useq_to_wseq 0 (1,"linear")) == 6
+wseq_end :: Num t => Wseq t a -> t
+wseq_end = snd . wseq_tspan
+
 -- * Duration
 
 dseq_dur :: Num t => Dseq t a -> t
@@ -94,14 +108,51 @@ wseq_dur = uncurry subtract . wseq_tspan
 
 -- * Window
 
--- | Keep only elements in the indicated temporal window.
+-- | Prefix of sequence where the start time precedes or is at the
+-- indicate time.
+wseq_until :: Ord t => t -> Wseq t a -> Wseq t a
+wseq_until tm = takeWhile (\((t0,_),_) -> t0 <= tm)
+
+-- | Keep only elements that are entirely contained within the indicated
+-- temporal window, which is inclusive at the left & right
+-- edges, ie. [t0,t1].  Halts processing at end of window.
 --
 -- > let r = [((5,1),'e'),((6,1),'f'),((7,1),'g'),((8,1),'h')]
--- > in wseq_twindow (5,9) (zip (zip [1..10] (repeat 1)) ['a'..]) == r
+-- > in wseq_twindow (5,9) (zip (zip [1..] (repeat 1)) ['a'..]) == r
+--
+-- > wseq_twindow (1,2) [((1,1),'a'),((1,2),'b')] == [((1,1),'a')]
 wseq_twindow :: (Num t, Ord t) => (t,t) -> Wseq t a -> Wseq t a
 wseq_twindow (w0,w1) =
     let f (st,du) = w0 <= st && (st + du) <= w1
-    in wseq_tfilter f
+    in wseq_tfilter f . wseq_until w1
+
+-- | Select nodes that are active at indicated time, comparison is
+-- inclusive at left and exclusive at right.  Halts processing at end
+-- of window.
+--
+-- > let sq = [((1,1),'a'),((1,2),'b')]
+-- > in map (wseq_at sq) [1,2] == [sq,[((1,2),'b')]]
+--
+-- > wseq_at (zip (zip [1..] (repeat 1)) ['a'..]) 3 == [((3,1),'c')]
+wseq_at :: (Num t,Ord t) => Wseq t a -> t -> Wseq t a
+wseq_at sq tm =
+    let sel ((t0,t1),_) = t0 <= tm && tm < (t0 + t1)
+        end ((t0,_),_) = t0 <= tm
+    in filter sel (takeWhile end sq)
+
+-- | Select nodes that are active within the indicated window, comparison is
+-- inclusive at left and exclusive at right.  Halts processing at end
+-- of window.
+--
+-- > let sq = [((0,2),'a'),((0,4),'b'),((2,4),'c')]
+-- > in wseq_at_window sq (1,3) == sq
+--
+-- > wseq_at_window (zip (zip [1..] (repeat 1)) ['a'..]) (3,4) == [((3,1),'c'),((4,1),'d')]
+wseq_at_window :: (Num t, Ord t) => Wseq t a -> (t,t) -> Wseq t a
+wseq_at_window sq (w0,w1) =
+    let f (t0,t1) t = t0 <= t && t < t1
+        g (st,du) = let w = (st,st + du) in f w w0 || f w w1
+    in wseq_tfilter g (wseq_until w1 sq)
 
 -- * Append
 
@@ -140,11 +191,24 @@ tseq_merge_resolve f =
         g (t,p) (_,q) = (t,f p q)
     in T.merge_by_resolve g cmp
 
+-- | Compare first by start time, then by duration.
+w_compare :: Ord t => ((t,t),a) -> ((t,t),a) -> Ordering
+w_compare ((t1,d1),_) ((t2,d2),_) =
+    case compare t1 t2 of
+      EQ -> compare d1 d2
+      r -> r
+
+-- | Merge considering only start times.
 wseq_merge :: Ord t => Wseq t a -> Wseq t a -> Wseq t a
 wseq_merge = O.mergeBy (compare `on` (fst . fst))
 
+-- | Merge set considering both start times & durations.
+wseq_merge_set :: Ord t => [Wseq t a] -> Wseq t a
+wseq_merge_set = T.merge_set_by w_compare
+
 -- * Lookup
 
+-- | Locate nodes to the left and right of indicated time.
 tseq_lookup_window_by :: (t -> t -> Ordering) -> Tseq t e -> t -> (Maybe (t,e),Maybe (t,e))
 tseq_lookup_window_by cmp =
     let recur l sq t =
@@ -276,7 +340,7 @@ wseq_tmap_dur f = let g (t,d) = (t,f d) in seq_tmap g
 seq_partition :: Ord v => (a -> v) -> [(t,a)] -> [(v,[(t,a)])]
 seq_partition voice sq =
     let assign m (t,a) = M.insertWith (++) (voice a) [(t,a)] m
-        from_map = sortBy (compare `on` fst) .
+        from_map = sortOn fst .
                    map (\(v,l) -> (v,reverse l)) .
                    M.toList
     in from_map (foldl assign M.empty sq)
@@ -379,6 +443,9 @@ group_f cmp =
 --
 -- > let r = [(0,"a"),(1,"bc"),(2,"de"),(3,"f")]
 -- > in tseq_group (zip [0,1,1,2,2,3] ['a'..]) == r
+--
+-- > tseq_group [(1,'a'),(1,'b')] == [(1,"ab")]
+-- > tseq_group [(1,'a'),(2,'b'),(2,'c')] == [(1,"a"),(2,"bc")]
 tseq_group :: (Eq t,Num t) => Tseq t a -> Tseq t [a]
 tseq_group = group_f (==)
 
@@ -432,35 +499,66 @@ tseq_latch def sq t =
 
 -- * Wseq
 
+-- | Sort 'Wseq' by start time, 'Wseq' ought never to be out of
+-- order.
+--
+-- > wseq_sort [((3,1),'a'),((1,3),'b')] == [((1,3),'b'),((3,1),'a')]
+wseq_sort :: Ord t => Wseq t a -> Wseq t a
+wseq_sort = sortBy (compare `on` (fst . fst))
+
 -- | Transform 'Wseq' to 'Tseq' by discaring durations.
 wseq_discard_dur :: Wseq t a -> Tseq t a
 wseq_discard_dur = let f ((t,_),e) = (t,e) in map f
 
--- | Edit durations to ensure that notes don't overlap.  If the same
--- note is played simultaneously delete shorter note.  If a note
--- extends into a later note shorten duration (apply /d_fn/ to iot).
+wseq_overlap_f :: (Eq e,Ord t,Num t) =>
+                  (e -> e -> Bool) -> (t -> t) -> ((t,t),e) -> Wseq t e -> Maybe (Wseq t e)
+wseq_overlap_f eq_fn dur_fn ((t,d),a) sq =
+  case find (eq_fn a . snd) sq of
+    Nothing -> Nothing
+    Just ((t',d'),a') ->
+      if t == t'
+      then if d <= d'
+           then Just sq -- delete LHS
+           else Just (((t,d),a) : delete ((t',d'),a') sq) -- delete RHS
+      else if t' < t + d
+           then Just (((t,dur_fn (t' - t)),a) : sq) -- truncate LHS
+           else Nothing
+
+-- | Determine if sequence has overlapping equal nodes.
+wseq_has_overlaps :: (Ord t, Num t, Eq e) => (e -> e -> Bool) -> Wseq t e -> Bool
+wseq_has_overlaps eq_fn =
+  let recur sq =
+        case sq of
+          [] -> False
+          h:sq' ->
+            case wseq_overlap_f eq_fn id h sq' of
+              Nothing -> recur sq'
+              Just _ -> True
+    in recur
+
+
+{- | Edit durations to ensure that nodes don't overlap.  If equal nodes
+     begin simultaneously delete the shorter node.  If a node
+     extends into a later node shorten the initial duration (apply /dur_fn/ to iot).
+
+> let sq = [((0,1),'a'),((0,5),'a'),((1,5),'a'),((3,1),'a')]
+> let r = [((0,1),'a'),((1,2),'a'),((3,1),'a')]
+> wseq_has_overlaps (==) sq == True
+> wseq_remove_overlaps (==) id sq == r
+> wseq_has_overlaps (==) (wseq_remove_overlaps (==) id sq) == False
+
+-}
 wseq_remove_overlaps :: (Eq e,Ord t,Num t) =>
-                        (e -> e -> Bool) -> (t -> t) ->
-                        Wseq t e -> Wseq t e
-wseq_remove_overlaps eq_fn d_fn =
-    let go sq =
-            case sq of
-              [] -> []
-              ((t,d),a):sq' ->
-                  case find (eq_fn a . snd) sq' of
-                      Nothing -> ((t,d),a) : go sq'
-                      Just ((t',d'),a') ->
-                          if t == t'
-                          then if d <= d'
-                               then -- delete LHS
-                                   go sq'
-                               else -- delete RHS
-                                   ((t,d),a) :
-                                   go (delete ((t',d'),a') sq')
-                          else if t' < t + d
-                               then ((t,d_fn (t' - t)),a) : go sq'
-                               else ((t,d),a) : go sq'
-    in go
+                        (e -> e -> Bool) -> (t -> t) -> Wseq t e -> Wseq t e
+wseq_remove_overlaps eq_fn dur_fn =
+  let recur sq =
+        case sq of
+          [] -> []
+          h:sq' ->
+            case wseq_overlap_f eq_fn dur_fn h sq' of
+              Nothing -> h : recur sq'
+              Just sq'' -> recur sq''
+    in recur
 
 -- | Unjoin elements (assign equal time stamps to all elements).
 seq_unjoin :: [(t,[e])] -> [(t,e)]
@@ -470,87 +568,149 @@ seq_unjoin = let f (t,e) = zip (repeat t) e in concatMap f
 wseq_unjoin :: Wseq t [e] -> Wseq t e
 wseq_unjoin = seq_unjoin
 
--- * On/Off
+-- | Shift (displace) onset times by /i/.
+--
+-- > wseq_shift 3 [((1,2),'a')] == [((4,2),'a')]
+wseq_shift :: Num t => t -> Wseq t a -> Wseq t a
+wseq_shift i = wseq_tmap_st (+ i)
 
--- | Container for values that have /on/ and /off/ modes.
-data On_Off a = On a | Off a deriving (Eq,Show)
+-- | Shift q to end of p and append.
+--
+-- > wseq_append [((1,2),'a')] [((1,2),'b')] == [((1,2),'a'),((4,2),'b')]
+wseq_append :: Num t => Wseq t a -> Wseq t a -> Wseq t a
+wseq_append p q = p ++ wseq_shift (wseq_end p) q
 
--- | Structural comparison at 'On_Off', 'On' compares less than 'Off'.
-cmp_on_off :: On_Off a -> On_Off b -> Ordering
-cmp_on_off p q =
+-- | 'foldl1' of 'wseq_append'
+--
+-- > wseq_concat [[((1,2),'a')],[((1,2),'b')]] == [((1,2),'a'),((4,2),'b')]
+wseq_concat :: Num t => [Wseq t a] -> Wseq t a
+wseq_concat = foldl1 wseq_append
+
+-- * Begin/End
+
+-- | Container to mark the /begin/ and /end/ of a value.
+data Begin_End a = Begin a | End a deriving (Eq,Show)
+
+-- | Functor instance.
+begin_end_map :: (t -> u) -> Begin_End t -> Begin_End u
+begin_end_map f x =
+    case x of
+      Begin a -> Begin (f a)
+      End a -> End (f a)
+
+-- | Structural comparison at 'Begin_End', 'Begin' compares less than 'End'.
+cmp_begin_end :: Begin_End a -> Begin_End b -> Ordering
+cmp_begin_end p q =
     case (p,q) of
-      (On _,Off _) -> LT
-      (On _,On _) -> EQ
-      (Off _,Off _) -> EQ
-      (Off _,On _) -> GT
+      (Begin _,End _) -> LT
+      (Begin _,Begin _) -> EQ
+      (End _,End _) -> EQ
+      (End _,Begin _) -> GT
 
 -- | Translate container types.
-either_to_on_off :: Either a a -> On_Off a
-either_to_on_off p =
+either_to_begin_end :: Either a a -> Begin_End a
+either_to_begin_end p =
     case p of
-      Left a -> On a
-      Right a -> Off a
+      Left a -> Begin a
+      Right a -> End a
 
 -- | Translate container types.
-on_off_to_either :: On_Off a -> Either a a
-on_off_to_either p =
+begin_end_to_either :: Begin_End a -> Either a a
+begin_end_to_either p =
     case p of
-      On a -> Left a
-      Off a -> Right a
+      Begin a -> Left a
+      End a -> Right a
 
--- | Convert 'Wseq' to 'Tseq' transforming elements to 'On' and 'Off'
--- parts.  When merging, /off/ elements precede /on/ elements at equal
--- times.
+begin_end_partition :: [Begin_End a] -> ([a],[a])
+begin_end_partition =
+  let f e (p,q) = case e of
+                    Begin x -> (x:p,q)
+                    End x -> (p,x:q)
+  in foldr f ([],[])
+
+-- | Add or delete element from accumulated state.
+begin_end_track :: Eq a => [a] -> Begin_End a -> [a]
+begin_end_track st e =
+  case e of
+    Begin x -> x : st
+    End x -> delete x st
+
+-- | Convert 'Wseq' to 'Tseq' transforming elements to 'Begin_End'.
+--   When merging, /end/ elements precede /begin/ elements at equal times.
 --
 -- > let {sq = [((0,5),'a'),((2,2),'b')]
--- >     ;r = [(0,On 'a'),(2,On 'b'),(4,Off 'b'),(5,Off 'a')]}
--- > in wseq_on_off sq == r
+-- >     ;r = [(0,Begin 'a'),(2,Begin 'b'),(4,End 'b'),(5,End 'a')]}
+-- > in wseq_begin_end sq == r
 --
 -- > let {sq = [((0,1),'a'),((1,1),'b'),((2,1),'c')]
--- >     ;r = [(0,On 'a'),(1,Off 'a')
--- >          ,(1,On 'b'),(2,Off 'b')
--- >          ,(2,On 'c'),(3,Off 'c')]}
--- > in wseq_on_off sq == r
-wseq_on_off :: (Num t, Ord t) => Wseq t a -> Tseq t (On_Off a)
-wseq_on_off sq =
-    let f ((t,d),a) = [(t,On a),(t + d,Off a)]
+-- >     ;r = [(0,Begin 'a'),(1,End 'a')
+-- >          ,(1,Begin 'b'),(2,End 'b')
+-- >          ,(2,Begin 'c'),(3,End 'c')]}
+-- > in wseq_begin_end sq == r
+wseq_begin_end :: (Num t, Ord t) => Wseq t a -> Tseq t (Begin_End a)
+wseq_begin_end sq =
+    let f ((t,d),a) = [(t,Begin a),(t + d,End a)]
         g l =
             case l of
               [] -> []
-              e:l' -> tseq_merge_by (T.ordering_invert .: cmp_on_off) e (g l')
+              e:l' -> tseq_merge_by (T.ord_invert .: cmp_begin_end) e (g l')
     in g (map f sq)
 
--- | 'on_off_to_either' of 'wseq_on_off'.
-wseq_on_off_either :: (Num t, Ord t) => Wseq t a -> Tseq t (Either a a)
-wseq_on_off_either = tseq_map on_off_to_either . wseq_on_off
+-- | 'begin_end_to_either' of 'wseq_begin_end'.
+wseq_begin_end_either :: (Num t, Ord t) => Wseq t a -> Tseq t (Either a a)
+wseq_begin_end_either = tseq_map begin_end_to_either . wseq_begin_end
 
--- | Variant that applies /on/ and /off/ functions to nodes.
+-- | Variant that applies /begin/ and /end/ functions to nodes.
 --
 -- > let {sq = [((0,5),'a'),((2,2),'b')]
 -- >     ;r = [(0,'A'),(2,'B'),(4,'b'),(5,'a')]}
--- > in wseq_on_off_f Data.Char.toUpper id sq == r
-wseq_on_off_f :: (Ord t,Num t) => (a -> b) -> (a -> b) -> Wseq t a -> Tseq t b
-wseq_on_off_f f g = tseq_map (either f g) . wseq_on_off_either
+-- > in wseq_begin_end_f Data.Char.toUpper id sq == r
+wseq_begin_end_f :: (Ord t,Num t) => (a -> b) -> (a -> b) -> Wseq t a -> Tseq t b
+wseq_begin_end_f f g = tseq_map (either f g) . wseq_begin_end_either
 
--- | Inverse of 'wseq_on_off' given a predicate function for locating
--- the /off/ node of an /on/ node.
+-- | Result for each time-point the triple (begin-list,end-list,hold-list).
+--   The elements of the end-list have been deleted from the hold list.
+tseq_begin_end_accum :: Eq a => Tseq t [Begin_End a] -> Tseq t ([a],[a],[a])
+tseq_begin_end_accum =
+  let f st (t,x) =
+            let (b,e) = begin_end_partition x
+                st' = foldl begin_end_track st x
+            in (st',(t,(b,e,st \\ e)))
+    in snd . mapAccumL f []
+
+tseq_accumulate :: Eq a => Tseq t [Begin_End a] -> Tseq t [a]
+tseq_accumulate =
+  let f st (t,e) =
+            let g st' = (st',(t,st'))
+            in g (foldl begin_end_track st e)
+    in snd . mapAccumL f []
+
+-- | The transition sequence of /active/ elements.
 --
--- > let {sq = [(0,On 'a'),(2,On 'b'),(4,Off 'b'),(5,Off 'a')]
+-- > let w = [((0,3),'a'),((1,2),'b'),((2,1),'c'),((3,3),'d')]
+-- > wseq_accumulate w == [(0,"a"),(1,"ba"),(2,"cba"),(3,"d"),(6,"")]
+wseq_accumulate :: (Eq a,Ord t,Num t) => Wseq t a -> Tseq t [a]
+wseq_accumulate = tseq_accumulate . tseq_group . wseq_begin_end
+
+-- | Inverse of 'wseq_begin_end' given a predicate function for locating
+-- the /end/ node of a /begin/ node.
+--
+-- > let {sq = [(0,Begin 'a'),(2,Begin 'b'),(4,End 'b'),(5,End 'a')]
 -- >     ;r = [((0,5),'a'),((2,2),'b')]}
--- > in tseq_on_off_to_wseq (==) sq == r
-tseq_on_off_to_wseq :: Num t => (a -> a -> Bool) -> Tseq t (On_Off a) -> Wseq t a
-tseq_on_off_to_wseq cmp =
+-- > in tseq_begin_end_to_wseq (==) sq == r
+tseq_begin_end_to_wseq :: Num t => (a -> a -> Bool) -> Tseq t (Begin_End a) -> Wseq t a
+tseq_begin_end_to_wseq cmp =
     let cmp' x e =
             case e of
-              Off x' -> cmp x x'
+              End x' -> cmp x x'
               _ -> False
         f e r = case seq_find (cmp' e) r of
-                        Nothing -> error "tseq_on_off_to_wseq: no matching off?"
+                        Nothing -> error "tseq_begin_end_to_wseq: no matching end?"
                         Just (t,_) -> t
         go sq = case sq of
                   [] -> []
-                  (_,Off _) : sq' -> go sq'
-                  (t,On e) : sq' -> let t' = f e sq' in ((t,t' - t),e) : go sq'
+                  (_,End _) : sq' -> go sq'
+                  (t,Begin e) : sq' -> let t' = f e sq' in ((t,t' - t),e) : go sq'
     in go
 
 -- * Interop
@@ -558,6 +718,9 @@ tseq_on_off_to_wseq cmp =
 useq_to_dseq :: Useq t a -> Dseq t a
 useq_to_dseq (t,e) = zip (repeat t) e
 
+useq_to_wseq :: Num t => t -> Useq t a -> Wseq t a
+useq_to_wseq t0 = dseq_to_wseq t0 . useq_to_dseq
+
 -- | The conversion requires a start time and a /nil/ value used as an
 -- /eof/ marker. Productive given indefinite input sequence.
 --
@@ -574,8 +737,8 @@ dseq_to_tseq t0 nil sq =
         a' = a ++ [nil]
     in zip t a'
 
--- | Variant where the /nil/ is take as the last element of the
--- sequence.
+-- | Variant where the /nil/ value is taken from the last element of
+-- the sequence.
 --
 -- > let r = zip [0,1,3,6,8,9] "abcdee"
 -- > in dseq_to_tseq_last 0 (zip [1,2,3,2,1] "abcde") == r
@@ -595,7 +758,8 @@ pseq_to_wseq t0 sq =
     in wseq_zip t d a
 
 -- | The last element of 'Tseq' is required to be an /eof/ marker that
--- has no duration and is not represented in the 'Dseq'.
+-- has no duration and is not represented in the 'Dseq'.  A 'nil'
+-- value is required in case the 'Tseq' does not begin at @0@.
 --
 -- > let r = zip [1,2,3,2,1] "abcde"
 -- > in tseq_to_dseq undefined (zip [0,1,3,6,8,9] "abcde|") == r
@@ -690,6 +854,30 @@ dseql_to_tseql =
             in (z',zip r el)
     in mapAccumL f
 
+-- * Cycle
+
+wseq_cycle' :: Num t => Wseq t a -> [Wseq t a]
+wseq_cycle' sq =
+    let (_,et) = wseq_tspan sq
+        t_sq = iterate (+ et) 0
+    in map (\x -> wseq_tmap (\(t,d) -> (x + t,d)) sq) t_sq
+
+-- | Only finite 'Wseq' can be cycled, the resulting Wseq is infinite.
+--
+-- > take 5 (wseq_cycle [((0,1),'a'),((3,3),'b')])
+wseq_cycle :: Num t => Wseq t a -> Wseq t a
+wseq_cycle = concat . wseq_cycle'
+
+-- | Variant cycling only /n/ times.
+--
+-- > wseq_cycle_n 3 [((0,1),'a'),((3,3),'b')]
+wseq_cycle_n :: Num t => Int -> Wseq t a -> Wseq t a
+wseq_cycle_n n = concat . take n . wseq_cycle'
+
+-- | 'wseq_until' of 'wseq_cycle'.
+wseq_cycle_until :: (Num t,Ord t) => t -> Wseq t a -> Wseq t a
+wseq_cycle_until et = wseq_until et . wseq_cycle
+
 -- * Type specialised map
 
 dseq_tmap :: (t -> t') -> Dseq t a -> Dseq t' a
